import java.util.NoSuchElementException;

/**
   A linked list is a sequence of links with efficient
   element insertion and removal. This class 
   contains a subset of the methods of the standard
   java.util.LinkedList class.
*/
public class LinkedList
{  
   /** 
      Constructs an empty linked list.
   */
   public LinkedList()
   {  
      first = null;
   }
   
   /**
      Returns the first element in the linked list.
      @return the first element in the linked list
   */
   public Object getFirst()
   {  
      if (first == null) 
         throw new NoSuchElementException();
      return first.data;
   }

   /**
      Removes the first element in the linked list.
      @return the removed element
   */
   public Object removeFirst()
   {  
      if (first == null) 
         throw new NoSuchElementException();
      Object obj = first.data;
      first = first.next;
      return obj;
   }

   /**
      Adds an element to the front of the linked list.
      @param obj the object to add
   */
   public void addFirst(Object obj)
   {  
      Link newLink = new Link();
      newLink.data = obj;
      newLink.next = first;
      first = newLink;
   }
   
   /**
      Returns an iterator for iterating through this list.
      @return an iterator for iterating through this list
   */
   public ListIterator listIterator()
   {  
      return new LinkedListIterator();
   }
   
   private Link first;
   
   private class Link
   {  
      public Object data;
      public Link next;
   }

   private class LinkedListIterator implements ListIterator
   {  
      /**
         Constructs an iterator that points to the front
         of the linked list.
      */
      public LinkedListIterator()
      {  
         position = null;
         previous = null;
      }
      
      /**
         Moves the iterator past the next element.
         @return the traversed element
      */
      public Object next()
      {  
         if (!hasNext())
            throw new NoSuchElementException();
         previous = position; // remember for remove

         if (position == null)
            position = first;
         else
            position = position.next;

         return position.data;
      }
      
      /**
         Tests if there is an element after the iterator 
         position.
         @return true if there is an element after the iterator 
         position
      */
      public boolean hasNext()
      {  
         if (position == null)
            return first != null;
         else
            return position.next != null;
      }
      
      /**
         Adds an element before the iterator position
         and moves the iterator past the inserted element.
         @param obj the object to add
      */
      public void add(Object obj)
      {  
         if (position == null)
         {
            addFirst(obj);
            position = first;
         }
         else
         {  
            Link newLink = new Link();
            newLink.data = obj;
            newLink.next = position.next;
            position.next = newLink;
            position = newLink;
         }
         previous = null;
      }
      
      /**
         Removes the last traversed element. This method may
         only be called after a call to the next() method.
      */
      public void remove()
      {  
         if (position == first)
         {
            removeFirst();
            position = first;
         }
         else 
         {  
            if (previous == null)
               throw new IllegalStateException();
            previous.next = position.next;
            position = previous;
         }
         previous = null;
      }

      /**
         Sets the last traversed element to a different 
         value. 
         @param obj the object to set
      */
      public void set(Object obj)
      {
         if (position == null)
            throw new NoSuchElementException();
         position.data = obj;
      }
            
      private Link position;
      private Link previous;
   }
}
